import os
import sys
import docx
import pandas as pd
from pdf2docx import Converter
from docx.oxml.table import CT_Tbl
from docx.oxml.text.paragraph import CT_P
from docx.table import Table
from docx.text.paragraph import Paragraph
from openpyxl.styles import Font, Border, Side, Alignment, PatternFill

# Set your working directory path
WORK_DIR = "/home/arslantste/public_html/pdf_to_excel"

def convert_pdf_to_docx(pdf_path, docx_path):
    cv = Converter(pdf_path)
    cv.convert(docx_path, start=0, end=None)
    cv.close()

def iter_block_items(parent):
    for child in parent.element.body.iterchildren():
        if isinstance(child, CT_P):
            yield Paragraph(child, parent)
        elif isinstance(child, CT_Tbl):
            yield Table(child, parent)

def style_sheet(ws, is_text_block=False):
    thin_border = Border(
        left=Side(style='thin'),
        right=Side(style='thin'),
        top=Side(style='thin'),
        bottom=Side(style='thin')
    )

    header_font = Font(bold=True, size=12)
    regular_font = Font(size=11)
    large_text_font = Font(size=13)
    heading_font = Font(bold=True, size=15)
    center_align = Alignment(vertical="center", wrap_text=True)
    header_fill = PatternFill(start_color="DDEEFF", end_color="DDEEFF", fill_type="solid")

    for row in ws.iter_rows():
        for cell in row:
            cell.border = thin_border
            cell.alignment = center_align

            if is_text_block:
                # Detect heading inside the first line of text block
                if cell.row == 1 and cell.column == 1 and isinstance(cell.value, str):
                    lines = cell.value.strip().split("\n")
                    if lines:
                        first_line = lines[0].strip()
                        if len(first_line.split()) <= 8 and first_line.isupper():
                            cell.font = heading_font
                        else:
                            cell.font = large_text_font
            else:
                if cell.row == 1:
                    cell.font = header_font
                    cell.fill = header_fill
                else:
                    cell.font = regular_font

    # Adjust column width based on content
    for col in ws.columns:
        max_length = 0
        column = col[0].column_letter
        for cell in col:
            try:
                if cell.value:
                    max_length = max(max_length, len(str(cell.value)))
            except:
                pass
        adjusted_width = min(max_length + 5, 80) if is_text_block else min(max_length + 2, 40)
        ws.column_dimensions[column].width = adjusted_width

def parse_docx_blocks(docx_path, output_excel):
    doc = docx.Document(docx_path)
    sheet_index = 1

    with pd.ExcelWriter(output_excel, engine='openpyxl') as writer:
        text_buffer = []

        for block in iter_block_items(doc):
            if isinstance(block, Table):
                # Flush text buffer before table
                if text_buffer:
                    text_joined = "\n".join(text_buffer).strip()
                    df_text = pd.DataFrame([[text_joined]], columns=["Content"])
                    sheet_name = f"Text{sheet_index}"
                    df_text.to_excel(writer, sheet_name=sheet_name, index=False)
                    style_sheet(writer.sheets[sheet_name], is_text_block=True)
                    text_buffer = []
                    sheet_index += 1

                # Handle table
                data = []
                for row in block.rows:
                    data.append([cell.text.strip() for cell in row.cells])
                df_table = pd.DataFrame(data)
                sheet_name = f"Table{sheet_index}"
                df_table.to_excel(writer, sheet_name=sheet_name, index=False)
                style_sheet(writer.sheets[sheet_name])
                sheet_index += 1

            elif isinstance(block, Paragraph):
                text = block.text.strip()
                if text:
                    text_buffer.append(text)

        # Flush remaining text buffer
        if text_buffer:
            text_joined = "\n".join(text_buffer).strip()
            df_text = pd.DataFrame([[text_joined]], columns=["Content"])
            sheet_name = f"Text{sheet_index}"
            df_text.to_excel(writer, sheet_name=sheet_name, index=False)
            style_sheet(writer.sheets[sheet_name], is_text_block=True)

    return True

def full_convert(pdf_path, output_excel):
    if not os.path.exists(pdf_path):
        print("pdf-not-found")
        return

    docx_temp = os.path.join(WORK_DIR, "temp.docx")
    convert_pdf_to_docx(pdf_path, docx_temp)

    if parse_docx_blocks(docx_temp, output_excel):
        print("success")
    else:
        print("fail")

    if os.path.exists(docx_temp):
        os.remove(docx_temp)

if __name__ == "__main__":
    if len(sys.argv) < 3:
        print("Usage: python pdf_to_excel_block_parser.py input.pdf output.xlsx")
    else:
        full_convert(sys.argv[1], sys.argv[2])
